"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.invokeLambda = void 0;
var client_lambda_1 = require("@aws-sdk/client-lambda");
var util_1 = require("./util");
var debug_1 = __importDefault(require("debug"));
var debug = (0, debug_1.default)('openapi-lambda-adapter');
var lambdaClient;
var getLambdaClient = function () {
    if (!lambdaClient) {
        lambdaClient = new client_lambda_1.Lambda({ region: 'eu-central-1' });
    }
    return lambdaClient;
};
var invokeLambda = function (params) { return __awaiter(void 0, void 0, void 0, function () {
    var payload, functionName, lambda, res, safePayload, response;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                payload = params.payload, functionName = params.functionName;
                lambda = getLambdaClient();
                return [4 /*yield*/, lambda
                        .invoke({
                        FunctionName: functionName,
                        InvocationType: 'RequestResponse',
                        LogType: 'None',
                        Payload: Buffer.from(payload)
                    })];
            case 1:
                res = _a.sent();
                debug('Lambda invocation response', res);
                if (!isSuccess(res.StatusCode)) {
                    return [2 /*return*/, toApiGwResp({
                            statusCode: 502,
                            body: { message: "Failed to invoke lambda ".concat(params.functionName, " synchronously. Check your IAM setup.") }
                        })];
                }
                safePayload = Buffer.from(res.Payload).toString('utf-8');
                if (hasTimeout(res, safePayload)) {
                    return [2 /*return*/, toApiGwResp({
                            statusCode: 504,
                            body: { message: 'Service timed out.', functionName: params.functionName, error: res.FunctionError, payload: safePayload }
                        })];
                }
                response = (0, util_1.safeParseJson)(safePayload);
                // response is not in JSON format
                if (!isApiGwStructuredResp(response)) {
                    return [2 /*return*/, toApiGwResp({
                            statusCode: 502,
                            body: {
                                message: 'Service returned a wrongly formatted response.',
                                functionName: params.functionName,
                                payload: safePayload,
                                payloadJson: response
                            }
                        })];
                }
                return [2 /*return*/, response];
        }
    });
}); };
exports.invokeLambda = invokeLambda;
var isSuccess = function (statusCode) { return statusCode >= 200 && statusCode < 300; };
var isApiGwStructuredResp = function (resp) {
    if (!resp || typeof resp !== 'object') {
        return false;
    }
    if (!('statusCode' in resp)) {
        return false;
    }
    return true;
};
var hasTimeout = function (res, safePayload) { return 'FunctionError' in res && (safePayload === null || safePayload === void 0 ? void 0 : safePayload.includes('Task timed out')); };
var toApiGwResp = function (params) {
    return {
        statusCode: params.statusCode,
        headers: {
            'Content-Type': 'application/json'
        },
        body: JSON.stringify(params.body)
    };
};
//# sourceMappingURL=lambda-invoker.js.map